#ifndef _SOUND_H_
#define _SOUND_H_

#include <mme/mme_api.h>

#define GET_SOUND_RATE_CODE(f) (((f)&0x0c)>>2)
#define BUFFER_SIZE 8192		// Size of MMOV buffer
#define CHAN_MAX 8			// Number of simultaneous sounds
#define BUFF_MAX CHAN_MAX*2		// Up to two buffers per sound

extern "C" {
void* my_mmeAllocBuffer(unsigned int size);
void* my_mmeAllocMem(unsigned int size);
void  my_mmeFreeMem(void * ptr);
void  my_mmeFreeBuffer(void * ptr);
void  audioCallback();
unsigned char my_mmeCheckForCallbacks();
void my_mmeProcessCallbacks();
void cvt55to11at8(char *inbuf, char *outbuf, DWORD length, WORD channels);
void cvt55to11at16(short *inbuf, short *outbuf, DWORD length, WORD channels);
void processMP3(long nbSamples,char *in, long inSize, char* out, long* outSize);
}

class Sound : public Character {
	WAVEFORMATEX	 wfx;
	char		*samples;	// Array of samples
	long		 nbSamples;	// Number of samples
	long		 flags;		// Copy of sound flags
	char		*streamBuffer;	// For accumulating streaming sounds
	long		 streamBufferSize;
	short		 delaySeek;	// MP3 specific variable
public:
	Sound(long id);
	~Sound();

	void		 setStream(long, char *, long);
	long		 getDelaySeek();
	void    	 setSoundFlags(long f);
	char		*setNbSamples(long n);
	char		*setNbSamplesMP3(long, short, char*, long);

	void		 convRate();
	long		 getFormat();
	long		 getRate();
	long		 getChannel();
	long		 getNbSamples();
	long		 getSampleSize();
	char		*getSamples();
};

struct SoundList {
	WAVEFORMATEX	 wfx;
	long	 	 nbSamples;
	long	 	 remaining;
	long	 	 inPoint, outPoint, loop, delaySeek;
	char		*current;

	SoundList	*next;
};



struct _channels {
	HWAVEOUT	 hWaveOut;	// Output channel handle
	UINT	 	 deviceID;	// Output channel ID
	UINT	 	 busy;	    	// Channel has active sound
	U32	 	 byteCount;	// Number of bytes in sound buffer
	U32	 	 loopCount;	// Number of times to play this sound
	MMTIME		*pos;		// Output channel current position
	WAVEFORMATEX	*wfx;		// Format of sound playing
};

struct _buffDesc {
	WAVEHDR		*waveHdr;	// buffer descriptors
	_channels	*channel;	// channel this buffer is playing on
	long	 	 alloc;		// actual size of buffer
	UINT	 	 busy;		// in-use flag
};

class SoundMixer {

static  long		 devices;	// Number of sound devices available
static	long		 nbInst;	// Number of instances
static	_channels	 *channels;
static	_buffDesc	 *buffDesc;

// Class variables
	SoundList	*list;
	_buffDesc	*SoundMixer::getBuffer(long *);

public:
	SoundMixer(char*);
	~SoundMixer();

	void		 startSound(Sound*, Control*);	// Register a sound to be played
	void		 stopSounds();		// Stop every current sounds in the instance

	long		 playSounds();		// Actually play sounds of all instances
};

#endif /* _SOUND_H_ */
